# TuXtremSplit - A Linux Xtremsplit file tool
#
# Copyright (C) 2011 zedtux <zedtux@zedroot.org>
#
# TuXtremSplit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# TuXtremSplit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TuXtremSplit; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
import logging
from tuxtremsplit import TuXtremSplit

from tuxtremsplit.application.filedescriptor import FileDescriptor
from tuxtremsplit.application.pathdescriptor import PathDescriptor
from tuxtremsplit.core.xtmgroup import XtmGroup
from tuxtremsplit.customexceptions import NoDescriptorError

class XtmPipeWorker(object):
    """
    XTM Pipe Worker
    Split or Join files from input to output.
    """

    def __init__(self):
        """ Constructor """
        TuXtremSplit().logger.debug("XtmPipeWorker#__init__()")
        self.__enqueued_descriptor = None
        self.__input = None
        self.__current_input = None
        self.__current_input_size = 0
        self.__output = None
        self.__current_output = None
        self.__current_output_size = 0
        self.__total_size = 0
        self.__buffer_size = 2048
        self.__output_filename = None

    def get_input(self):
        return self.__input
    input = property(get_input)

    def get_output(self):
        return self.__output
    output = property(get_output)

    def enqueue(self, descriptor):
        """ Should receive a FileDescriptor, PathDescriptor or XtmGroup """
        self.__enqueued_descriptor = descriptor
        return self

    def as_input(self):
        """ Place prepared descriptor as input """
        self._as(input=True)

    def as_output(self):
        """ Place prepared descriptor as output """
        self._as(input=False)

    def _as(self, input):
        if not self.__enqueued_descriptor:
            raise NoDescriptorError(
                "No descriptor ready to be placed as output."
            )

        TuXtremSplit().logger.debug(
            "Enqueuing %s as %s" % (
                self.__enqueued_descriptor.name,
                "input" if input else "output"
            )
        )
        if input:
            self.__input = self.__enqueued_descriptor
        else:
            self.__output = self.__enqueued_descriptor
        self._reset_enqueued_descriptor()

    def _reset_enqueued_descriptor(self):
        self.__enqueued_descriptor = None

    def _setup_is_one_to_many(self):
        """ Return True if input is an instance of class FileDescriptor """
        return isinstance(self.__input, FileDescriptor) or \
               isinstance(self.__input, PathDescriptor)

    def _setup_is_many_to_one(self):
        """ Return True if input is an instance of class XtmGroup """
        return isinstance(self.__input, XtmGroup)

    def _inputs_pop(self):
        """ Pop the first or the next input item """
        TuXtremSplit().logger.debug("Popuping input...")

        if self._setup_is_one_to_many():
            # Split a big file
            if not self.__current_input:
                TuXtremSplit().logger.debug("Initializing input...")
                return self.__input.open_file()
            else:
                return self.__current_input

        elif self._setup_is_many_to_one():
            # Joining xtm files
            # First call
            if not self.__current_input:
                # Return the .001.xtm or .001.exe file
                input = self.__input.main_file
            else:
                # Close the input file
                self.__current_input.close_file()
                # Reset readed size for next file
                self.__current_input_size = 0
                input = self.__input.next_file()

            if input:
                # Initialize the input file in order to use it directly
                input.open_file()

                # First call
                if not self.__current_input:
                    # Set the output filename from the XTM header
                    self.__output_filename = input.header.original_filename
                    if input.is_main_file and input.extension.is_exe:
                        # Do not copy the Xtremsplit SFX header data
                        input.move_after_xtm_sfx_header()
                    # Do not copy the header in the output file
                    input.move_after_xtm_header()
                    self.__current_input_size = input.file.tell()

                TuXtremSplit().logger.debug(
                    "Output filename is %s" % self.__output_filename
                )
                return input

    def _outputs_pop(self):
        """ Pop the first or the next output item """
        TuXtremSplit().logger.debug("Popuping output...")
        if self._setup_is_one_to_many():
            # Split a big file
            # First call
            if not self.__current_output:
                # Return the .001.xtm file
                output = self.__output.main_file
            else:
                # Write Xtremsplit SFX footer before to close the first xtm
                if self.__output.create_sfx and \
                   self.__current_output.is_main_file:
                    TuXtremSplit().logger.debug("Writting SFX footer...")
                    self.__current_output.write_sfx_footer()

                # Close the output file
                self.__current_output.close_file()
                self.__current_output_size = 0
                output = self.__output.next_file()

            if output:
                # Initialize the input file in order to use it directly
                output.open_file(write=True)

                # First call
                if not self.__current_output:
                    if self.__output.create_sfx:
                        TuXtremSplit().logger.debug("Writting SFX Header...")
                        output.write_sfx_header()
                    TuXtremSplit().logger.debug("Writting XTM Header...")
                    # Write the Xtm Header to the first file
                    output.write_xtm_header(self.__output)
                    self.__current_output_size = output.file.tell()

                return output

        elif self._setup_is_many_to_one():
            # Joining xtm files
            # First call
            if not self.__current_output:
                # Check and create the output file
                # from the header of the very first XTM file
                return self.output.open_file(self.__output_filename,
                                             write=True)
            else:
                return self.__current_output

    def _buffer_size(self):
        """
        Calculate the buffer size to use
        depending the mode of the pipe and the current file.
        
        - Ignore MD5 chechsums at the end of the last file
            when joining xtm files.
        """
        from tuxtremsplit.application import Application

        # When splitting file
        if self._setup_is_one_to_many():
            # Use normal buffer size while the current output size
            # is not too high
            if (self.__current_output_size + self.__buffer_size) < \
                self.__current_output.size_without_sfx:
                return self.__buffer_size
            else:
                new_buffer_size = (self.__current_output.size - 
                                   self.__current_output_size)
                if self.__current_output.is_main_file and \
                   self.__current_output.extension.is_exe:
                    new_buffer_size -= Application().settings\
                                                    .xtm_sfx.footer_size
                return new_buffer_size

        elif self._setup_is_many_to_one():

            # Only when joining an .001.exe file
            if self.__current_input.is_main_file and \
               self.__current_input.extension.is_exe:
                # Ignore the XTM SFX footer
                if (self.__current_input_size + self.__buffer_size) > \
                   (self.__current_input.size - Application().settings\
                                                             .xtm_sfx\
                                                             .footer_size):
                    return (self.__current_input.size - \
                            self.__current_input_size - \
                            Application().settings.xtm_sfx.footer_size)
            
            # Only when current file is the last one from the group
            elif self.input.is_the_last_file(self.__current_input):
                # Last file may contains the MD5 checksums of XTM parts
                if self.input.main_file.header.md5_enabled:
                    # Only when arriving near the end of the last file
                    if (self.__current_input_size + self.__buffer_size) > \
                        (self.__current_input.size - (
                            32 * self.input.main_file.header.number_of_parts)):
                        return (self.__current_input.size - (32 * \
                                self.input.main_file.header.number_of_parts) -\
                                self.__current_input_size)
        return self.__buffer_size

    def _rotate(self):
        """ Rotate Inputs and Outputs """
        TuXtremSplit().logger.debug("Rotating inputs / outputs...")
        # Input file
        before_rotation = self.__current_input
        self.__current_input = self._inputs_pop()
        if self.__current_input and self.__current_input != before_rotation:
            TuXtremSplit().logger.info("In: %s" % self.__current_input.name)

        # Output file
        before_rotation = self.__current_output
        self.__current_output = self._outputs_pop()
        if self.__current_output and self.__current_output != before_rotation:
            TuXtremSplit().logger.info("Out: %s" % self.__current_output.name)

        return (self.__current_input and self.__current_output)

    def start(self):
        """ Execute the action """
        self.inspect()

        data = None
        while True:

            # Read a block of bytes from the current input
            if self.__current_input:
                data = self.__current_input.file.read(self._buffer_size())

            # End of file from current input
            if not data:
                # No more input or output file
                if not self._rotate():
                    TuXtremSplit().logger.info("Finished")
                    # End of worker
                    break
                else:
                    TuXtremSplit().logger.debug("Input starting point %d" %
                        self.__current_input.file.tell())

            if data:
                # Write block of bytes from the current input
                # to the current output
                self.__current_output.file.write(data)

                # Update sizes
                self.__current_input_size += len(data)
                self.__current_output_size += len(data)
                self.__total_size += len(data)

                if (self._setup_is_one_to_many() and \
                    self.__current_output_size % 5000 == 0) or \
                   (self._setup_is_many_to_one() and \
                    self.__current_input_size % 5000 == 0):
                    TuXtremSplit().logger.debug(
                        "Readed size: Current: %d | Total: %d" % (
                            self.__current_input_size, self.__total_size
                        )
                    )

        self.after_finished()

    def inspect(self):
        """ Output when debug is enabled some details about the Pipe """
        if self._setup_is_one_to_many():
            TuXtremSplit().logger.debug("1 input and %d outputs" %
                                        self.__output.size)
        elif self._setup_is_many_to_one():
            TuXtremSplit().logger.debug("%s inputs and 1 output" %
                                        self.__input.size)

    def after_finished(self):
        """ Fired method after treatment done """
        from tuxtremsplit.application import Application
        if self._setup_is_one_to_many() and Application().arguments.md5:
            self.__output.write_md5_sums()
